"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createServerConfig = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const webpack_1 = __importDefault(require("webpack"));
const webpack_node_externals_1 = __importDefault(require("webpack-node-externals"));
const tsconfig_paths_webpack_plugin_1 = __importDefault(require("tsconfig-paths-webpack-plugin"));
const getCSSModuleLocalIdent_1 = __importDefault(require("react-dev-utils/getCSSModuleLocalIdent"));
const app_configs_1 = __importDefault(require("./app-configs"));
const babel_server_1 = __importDefault(require("./babel-server"));
const postcss_1 = __importDefault(require("./postcss"));
const get_entry_1 = __importDefault(require("./util/get-entry"));
const babel_dependencies_1 = require("./babel-dependencies");
const server_externals_exemptions_1 = require("./server-externals-exemptions");
const run_script_webpack_plugin_1 = require("run-script-webpack-plugin");
const case_sensitive_paths_webpack_plugin_1 = __importDefault(require("case-sensitive-paths-webpack-plugin"));
const ReloadServerPlugin = require('../plugins/reload-server-webpack-plugin');
const WatchMissingNodeModulesPlugin = require('../plugins/watch-missing-node-modules-plugin');
const assetsIgnoreBanner = fs_1.default.readFileSync(require.resolve('./util/node-assets-ignore'), 'utf8');
const sourceMapSupportBanner = fs_1.default.readFileSync(require.resolve('./util/install-sourcemap-support'), 'utf8');
// style files regexes
const cssRegex = /\.css$/;
const cssModuleRegex = /\.module\.css$/;
function getSingleEntry(entryPoint, mode) {
    const prefix = (mode === 'dev' && app_configs_1.default.useServerHMR) ? [`${require.resolve('webpack/hot/poll')}?1000`] : [];
    return [...prefix, ...entryPoint];
}
// This is a function that wil return webpack configuration to build a server
const createServerConfig = (mode) => ({
    mode: mode === 'dev' ? 'development' : 'production',
    // fail on first error in production mode
    bail: mode === 'prod',
    devtool: mode === 'dev' ? app_configs_1.default.devSourceMaps : 'source-map',
    target: 'node',
    node: {
        __filename: true,
        __dirname: true
    },
    entry: (0, get_entry_1.default)(app_configs_1.default.serverEntry, getSingleEntry, mode),
    context: app_configs_1.default.cwd,
    output: {
        assetModuleFilename: 'static/media/[name].[hash:8][ext]',
        // Add /* filename */ comments to generated require()s in the output.
        pathinfo: true,
        path: app_configs_1.default.serverOutputPath,
        publicPath: app_configs_1.default.publicPath,
        filename: app_configs_1.default.serverOutput,
        chunkFilename: '[name].js',
        // Point sourcemap entries to original disk location (format as URL on Windows)
        devtoolModuleFilenameTemplate: (info) => path_1.default
            .relative(app_configs_1.default.appSrc, info.absoluteResourcePath)
            .replace(/\\/g, '/'),
    },
    cache: mode === 'dev' ? {
        type: 'filesystem',
        name: 'server',
        buildDependencies: {
            config: [__filename],
        },
    } : false,
    externalsPresets: { node: true },
    externals: [(0, webpack_node_externals_1.default)({
            allowlist: server_externals_exemptions_1.serverExternalsExemptions,
            // we cannot determine node_modules location before arui-scripts installation, so just load
            // dependencies list from package.json
            modulesFromFile: true,
        })],
    optimization: {
        minimize: false,
        nodeEnv: mode === 'dev' ? false : 'production',
    },
    resolve: {
        // This allows you to set a fallback for where Webpack should look for modules.
        // We placed these paths second because we want `node_modules` to "win"
        // if there are any conflicts. This matches Node resolution mechanism.
        // https://github.com/facebookincubator/create-react-app/issues/253
        modules: ['node_modules', app_configs_1.default.appNodeModules],
        // These are the reasonable defaults supported by the Node ecosystem.
        // We also include JSX as a common component filename extension to support
        // some tools, although we do not recommend using it, see:
        // https://github.com/facebookincubator/create-react-app/issues/290
        // `web` extension prefixes have been added for better support
        // for React Native Web.
        extensions: ['.web.js', '.mjs', '.js', '.json', '.web.jsx', '.jsx', '.ts', '.tsx'],
        plugins: ([
            (app_configs_1.default.tsconfig && new tsconfig_paths_webpack_plugin_1.default({
                configFile: app_configs_1.default.tsconfig,
                extensions: ['.web.js', '.mjs', '.js', '.json', '.web.jsx', '.jsx', '.ts', '.tsx']
            }))
        ].filter(Boolean)),
    },
    module: {
        // typescript interface will be removed from modules, and we will get an error on correct code
        // see https://github.com/webpack/webpack/issues/7378
        strictExportPresence: !app_configs_1.default.tsconfig,
        rules: [
            {
                oneOf: ([
                    // Process JS with Babel.
                    {
                        test: app_configs_1.default.useTscLoader ? /\.(js|jsx|mjs)$/ : /\.(js|jsx|mjs|ts|tsx)$/,
                        include: app_configs_1.default.appSrc,
                        loader: require.resolve('babel-loader'),
                        options: Object.assign(Object.assign({}, babel_server_1.default), { 
                            // This is a feature of `babel-loader` for webpack (not Babel itself).
                            // It enables caching results in ./node_modules/.cache/babel-loader/
                            // directory for faster rebuilds.
                            cacheDirectory: mode === 'dev' }),
                    },
                    (app_configs_1.default.tsconfig && app_configs_1.default.useTscLoader) && {
                        test: /\.tsx?$/,
                        use: [
                            {
                                loader: require.resolve('babel-loader'),
                                options: Object.assign({
                                    // This is a feature of `babel-loader` for webpack (not Babel itself).
                                    // It enables caching results in ./node_modules/.cache/babel-loader/
                                    // directory for faster rebuilds.
                                    cacheDirectory: mode === 'dev'
                                }, babel_server_1.default)
                            },
                            {
                                loader: require.resolve('ts-loader'),
                                options: {
                                    onlyCompileBundledFiles: true,
                                    transpileOnly: true,
                                    happyPackMode: true,
                                    configFile: app_configs_1.default.tsconfig
                                }
                            }
                        ]
                    },
                    // Process any JS outside of the app with Babel.
                    // Unlike the application JS, we only compile the standard ES features.
                    {
                        test: /\.(js|mjs)$/,
                        exclude: /@babel(?:\/|\\{1,2})runtime/,
                        loader: require.resolve('babel-loader'),
                        options: Object.assign(Object.assign({}, babel_dependencies_1.babelDependencies), { babelrc: false, configFile: false, compact: false, cacheDirectory: mode === 'dev', cacheCompression: false }),
                    },
                    // replace css imports with empty files
                    {
                        test: cssRegex,
                        exclude: cssModuleRegex,
                        loader: require.resolve('null-loader')
                    },
                    {
                        test: cssModuleRegex,
                        use: [
                            {
                                loader: require.resolve('css-loader'),
                                options: {
                                    modules: {
                                        exportOnlyLocals: true,
                                        getLocalIdent: getCSSModuleLocalIdent_1.default
                                    },
                                },
                            },
                            {
                                loader: require.resolve('postcss-loader'),
                                options: {
                                    postcssOptions: {
                                        plugins: postcss_1.default,
                                    }
                                },
                            },
                        ],
                    },
                    {
                        exclude: [/\.(js|jsx|mjs|ts|tsx)$/, /\.(html|ejs)$/, /\.json$/],
                        type: 'asset/resource',
                        generator: {
                            outputPath: app_configs_1.default.publicPath,
                            publicPath: app_configs_1.default.publicPath,
                        }
                    },
                ].filter(Boolean)),
            },
        ],
    },
    plugins: ([
        new webpack_1.default.BannerPlugin({
            banner: assetsIgnoreBanner,
            raw: true,
            entryOnly: false
        }),
        app_configs_1.default.installServerSourceMaps && new webpack_1.default.BannerPlugin({
            banner: sourceMapSupportBanner,
            raw: true,
            entryOnly: false
        }),
        // dev plugins:
        mode === 'dev' && (app_configs_1.default.useServerHMR
            ? new run_script_webpack_plugin_1.RunScriptWebpackPlugin({
                name: app_configs_1.default.serverOutput
            })
            : new ReloadServerPlugin({ script: path_1.default.join(app_configs_1.default.serverOutputPath, app_configs_1.default.serverOutput) })),
        mode === 'dev' && new webpack_1.default.NoEmitOnErrorsPlugin(),
        // Watcher doesn't work well if you mistype casing in a path so we use
        // a plugin that prints an error when you attempt to do this.
        // See https://github.com/facebookincubator/create-react-app/issues/240
        mode === 'dev' && new case_sensitive_paths_webpack_plugin_1.default(),
        // If you require a missing module and then `npm install` it, you still have
        // to restart the development server for Webpack to discover it. This plugin
        // makes the discovery automatic so you don't have to restart.
        // See https://github.com/facebookincubator/create-react-app/issues/186
        mode === 'dev' && new WatchMissingNodeModulesPlugin(app_configs_1.default.appNodeModules),
        mode === 'dev' && app_configs_1.default.useServerHMR && new webpack_1.default.HotModuleReplacementPlugin()
    ].filter(Boolean)),
    experiments: {
        backCompat: app_configs_1.default.webpack4Compatibility,
    },
    // Без этого комиляция трирегилась на изменение в node_modules и приводила к утечке памяти
    watchOptions: {
        ignored: new RegExp(app_configs_1.default.watchIgnorePath.join('|')),
        aggregateTimeout: 100,
    },
    performance: {
        // we don't really care about bundle size for server
        hints: false,
    },
});
exports.createServerConfig = createServerConfig;
